import React, { ComponentProps, FC, useCallback, useMemo } from "react";
import { Select, Tag } from "udesk-ui";

export enum modeEnums {
    Multiple = 'multiple',
}

type AgentType = {
    label: string;
    nick_name: string;
    value: number;
    id: number;
}

type PropsInterface = {
    disabled?: Boolean,
    agents: AgentType[],
    valueMode: modeEnums, // 因组件默认多选，想单选需传入''
    readOnly?:boolean,
    value: string,
    placeholder?: string,
    onChange?: (value: string, options: object[]) => void,
} & Omit<ComponentProps<typeof Select>, 'mode' | 'onChange' | 'value'>;

const AgentSelect: FC<PropsInterface> = React.memo((props) => {
    const { agents = [], valueMode = modeEnums.Multiple, value, onChange = () => {}, disabled, readOnly, placeholder, ...restProps } = props;
    const options = useMemo<AgentType[]>(() => agents.map(i => ({ ...i, value: i.id, label: i.nick_name })), [agents]);

    let isMulValue = useMemo(() => {
        return valueMode === modeEnums.Multiple;
    }, [valueMode]);

    const selectValue: number[] = useMemo(() => {
        // 无论单选多选都转换为数组做处理
        let formattedValueArr: any[];
        if (typeof value === 'string' && value) {
            formattedValueArr = value.split(',');
        } else if (typeof value === 'number') {
            formattedValueArr = [value];
        } else if (Array.isArray(value)) {
            formattedValueArr = value;
        } else {
            formattedValueArr = [];
        }

        // value需要和option的值是同一种类型
        return formattedValueArr?.map(value => value && isNaN(Number(value)) ? value : Number(value));
    }, [isMulValue, value])

    const selectData: any[] = useMemo(() => {
        const selectOptions = options.filter(item => {
            return selectValue.includes(item.value);
        });
        return selectOptions;
    }, [isMulValue, selectValue]);

    const handleChange = useCallback((value, options) => {
        onChange(formatValueToString(value), options);
    }, [])

    const handleFilter = useCallback((input, option) => option?.nick_name.toLowerCase().indexOf(input.toLowerCase()) > -1 || option?.agent_name_pinyin.toLowerCase().indexOf(input.toLowerCase()) > -1, []);

    const readOnlyEle = useMemo(() => {
        if (!selectData.length) {
            return <span>- -</span>
        }
        return selectData.map(item => {
            if (isMulValue) {
                return <Tag className="custom-tag">{item?.label || ''}</Tag>
            }
            return <span>{item?.label || ''}</span>
        });
    }, [selectData, isMulValue]);

    if (readOnly) {
        return <React.Fragment>{readOnlyEle}</React.Fragment>;
    }

    return (
        <Select
            {...restProps}
            showSearch
            allowClear
            style={{ width: '100%' }}
            mode={valueMode}
            disabled={disabled}
            value={selectValue}
            placeholder={placeholder}
            onChange={handleChange}
            filterOption={handleFilter}
            options={options} 
        />
    )
});

AgentSelect.defaultProps = {
    disabled: false,
    readOnly: false,
    dropdownMatchSelectWidth: false,
    onChange: () => {},
};

// 为了兼容原有数据结构,原数据格式为字符串(自定义字段使用)
function formatValueToString(value: string | number[]) {
    if (Array.isArray(value)) {
        return value.join(',');
    }
    return value;
}

export default AgentSelect;
